/* LAN9118End.h - SMSC LAN9118 Family driver header */ 

/* Copyright (c) 2005-2006 SMSC */

/*
modification history
--------------------

12/18/2005 (v0.2) - SC
Debugged driver for PIO operation. Driver passes SmartBits NIC tests and 
survived SmartBits line-rate input traffic.

11/22/2005 (v0.1) - Phong Le
Ported from Linux version 1.21

Version: $Author$ $Id$ $DateTime$ $Change$

*/

#ifndef __INCLAN9118Endh
#define __INCLAN9118Endh

#ifdef __cplusplus
extern "C" {
#endif
/* ioctl Commands*/
#define COMMAND_GET_SIGNATURE 		_IOR('e',130, NULL)
#define COMMAND_GET_FLOW_PARAMS		_IOR('e',131,SMSC9118_IOCTL_DATA *)
#define COMMAND_DUMP_EEPROM			_IOR('e',132,SMSC9118_IOCTL_DATA *)
#define COMMAND_GET_CONFIGURATION	_IOR('e',133, LAN9118_END_DEV **)
#define COMMAND_GET_MAC_ADDRESS		_IOR('e',134, SMSC9118_IOCTL_DATA *)
#define COMMAND_LOAD_MAC_ADDRESS	_IOR('e',135, SMSC9118_IOCTL_DATA *)
#define COMMAND_GET_LINK_MODE		_IOR('e',136, SMSC9118_IOCTL_DATA *)
#define COMMAND_MAC_GET_REG			_IOR('e',137, SMSC9118_IOCTL_DATA *)
#define COMMAND_DUMP_PHY_REGS		_IOR('e',138, SMSC9118_IOCTL_DATA *)
#define COMMAND_PHY_GET_REG			_IOR('e',139, SMSC9118_IOCTL_DATA *)
#define COMMAND_DUMP_LAN_REGS		_IOR('e',140, SMSC9118_IOCTL_DATA *)
#define COMMAND_DUMP_MAC_REGS		_IOR('e',141, SMSC9118_IOCTL_DATA *)
#define COMMAND_CHECK_LINK			_IOR('e',142,NULL)
#define COMMAND_GET_AUTOMDIX		_IOR('e',143, SMSC9118_IOCTL_DATA *)



#define 	COMMAND_SET_FLOW_PARAMS		_IOW('e',151,SMSC9118_IOCTL_DATA *)
#define 	COMMAND_SET_MAC_ADDRESS		_IOW('e',152,SMSC9118_IOCTL_DATA *)
#define 	COMMAND_SAVE_MAC_ADDRESS	_IOW('e',153,SMSC9118_IOCTL_DATA *)
#define 	COMMAND_SET_DEBUG_MODE		_IOW('e',154,SMSC9118_IOCTL_DATA *)
#define 	COMMAND_SET_LINK_MODE		_IOW('e',155,SMSC9118_IOCTL_DATA *)
#define 	COMMAND_MAC_SET_REG			_IOW('e',156,SMSC9118_IOCTL_DATA *)
#define 	COMMAND_PHY_SET_REG			_IOW('e',157,SMSC9118_IOCTL_DATA *)
#define 	COMMAND_SET_AUTOMDIX		_IOR('e',158, SMSC9118_IOCTL_DATA *)







/* driver flags */
typedef UINT32 LAN9118_DRV_FLAG;

#define LAN9118_FLAG_POLL	0x01	/* for poll mode */    

#define LAN9118_WIDE_MASK	0x01	/* option bit 0 (LSB) */
#define LAN9118_INTERVAL_MASK	0x02	/* option bit 1 */
#define MINPKTSIZE		64	/* Minimum packet size */
#define LAN9118_PAGE_SIZE	256	/* Page size in buffer memory */
#define LAN9118_TX_POOL_SIZE	1514	/* TX pool size */

/* */ #undef USE_LED1_WORK_AROUND /* #define	USE_LED1_WORK_AROUND	/* 10/100 LED link-state inversion */
#define	USE_PHY_WORK_AROUND	/* output polarity inversion */

#define HIBYTE(word)  ((UINT8)(((UINT16)(word))>>8))
#define LOBYTE(word)  ((UINT8)(((UINT16)(word))&0x00FFU))
#define HIWORD(dWord) ((UINT16)(((UINT32)(dWord))>>16))
#define LOWORD(dWord) ((UINT16)(((UINT32)(dWord))&0x0000FFFFUL))

#define TRANSFER_PIO			(256UL)
#define TRANSFER_REQUEST_DMA	(254UL)

/*******************************************************
* Macro: SMSC_TRACE
* Description: 
*    This macro is used like printf. 
*    It can be used anywhere you want to display information
*    For any release version it should not be left in 
*      performance sensitive Tx and Rx code paths.
*    To use this macro define USE_TRACE and set bit 0 of debug_mode
*******************************************************/
#define USE_TRACE

#ifdef USING_LINT
extern void SMSC_TRACE(const char * a, ...);
#else /*not USING_LINT */
#ifdef USE_TRACE
extern UINT32 debug_mode;
#ifndef USE_WARNING
#define USE_WARNING
#endif
#define SMSC_TRACE(msg)											\
		DRV_LOG(LAN9118_DEBUG_TRACE, "SMSC: %s\n", (UINT32)msg,		\
		2,3,4,5,6);
#else
#define SMSC_TRACE(msg,args...)
#endif
#endif /*not USING_LINT */

/*******************************************************
* Macro: SMSC_WARNING
* Description: 
*    This macro is used like printf. 
*    It can be used anywhere you want to display warning information
*    For any release version it should not be left in 
*      performance sensitive Tx and Rx code paths.
*    To use this macro define USE_TRACE or 
*      USE_WARNING and set bit 1 of debug_mode
*******************************************************/

#define USE_WARNING

#ifdef USE_WARNING
	#define SMSC_WARNING(msg)													\
		DRV_LOG(LAN9118_DEBUG_WARNING, "SMSC_WARNING: %s\n", (UINT32)msg,2,3,4,5,6)
#else
	#define SMSC_WARNING(msg)
#endif


/*******************************************************
* Macro: SMSC_ASSERT
* Description: 
*    This macro is used to test assumptions made when coding. 
*    It can be used anywhere, but is intended only for situations
*      where a failure is fatal. 
*    If code execution where allowed to continue it is assumed that 
*      only further unrecoverable errors would occur and so this macro
*      includes an infinite loop to prevent further corruption.
*    Assertions are only intended for use during developement to 
*      insure consistency of logic through out the driver.
*    A driver should not be released if assertion failures are 
*      still occuring.
*    To use this macro define USE_TRACE or USE_WARNING or
*      USE_ASSERT
*******************************************************/

#define USE_ASSERT	/* sc */

#ifdef USING_LINT
extern void SMSC_ASSERT(UINT32 condition);
#else /*not USING_LINT */
#ifdef USE_ASSERT
#	define SMSC_ASSERT(condition)											\
	if(!(condition)) {														\
		DRV_LOG(LAN9118_DEBUG_ASSERT, "SMSC_ASSERTION_FAILURE: Line=%d\n", 	\
		__LINE__, 2, 3, 4, 5, 6);											\
	}
#else 
#	define SMSC_ASSERT(condition)
#endif
#endif /*not USING_LINT */


/*Below are the register offsets and bit definitions
  of the LAN9118 memory space
*/
#define RX_DATA_FIFO	    (0x00UL)

#define TX_DATA_FIFO        (0x20UL)
#define		TX_CMD_A_INT_ON_COMP_		(0x80000000UL)
#define		TX_CMD_A_INT_BUF_END_ALGN_	(0x03000000UL)
#define		TX_CMD_A_INT_4_BYTE_ALGN_	(0x00000000UL)
#define		TX_CMD_A_INT_16_BYTE_ALGN_	(0x01000000UL)
#define		TX_CMD_A_INT_32_BYTE_ALGN_	(0x02000000UL)
#define		TX_CMD_A_INT_DATA_OFFSET_	(0x001F0000UL)
#define		TX_CMD_A_INT_FIRST_SEG_		(0x00002000UL)
#define		TX_CMD_A_INT_LAST_SEG_		(0x00001000UL)
#define		TX_CMD_A_BUF_SIZE_			(0x000007FFUL)
#define		TX_CMD_B_PKT_TAG_			(0xFFFF0000UL)
#define		TX_CMD_B_ADD_CRC_DISABLE_	(0x00002000UL)
#define		TX_CMD_B_DISABLE_PADDING_	(0x00001000UL)
#define		TX_CMD_B_PKT_BYTE_LENGTH_	(0x000007FFUL)

#define RX_STATUS_FIFO      (0x40UL)
#define		RX_STS_ES_				(0x00008000UL)
#define		RX_STS_MCAST_			(0x00000400UL)
#define 		RX_STATUS_FIFO_PEEK 	(0x44UL)
#define 		TX_STATUS_FIFO		(0x48UL)
#define 		TX_STATUS_FIFO_PEEK 	(0x4CUL)
#define ID_REV              (0x50UL)
#define		ID_REV_CHIP_ID_		(0xFFFF0000UL)	/* RO */
#define		ID_REV_REV_ID_		(0x0000FFFFUL)	/* RO */

#define INT_CFG				(0x54UL)
#define		INT_CFG_INT_DEAS_	(0xFF000000UL)	/* R/W */
#define		INT_CFG_IRQ_INT_	(0x00001000UL)	/* RO */
#define		INT_CFG_IRQ_EN_		(0x00000100UL)	/* R/W */
#define		INT_CFG_IRQ_POL_	(0x00000010UL)	/* R/W Not Affected by SW Reset */
#define		INT_CFG_IRQ_TYPE_	(0x00000001UL)	/* R/W Not Affected by SW Reset */

#define INT_STS				(0x58UL)
#define		INT_STS_SW_INT_		(0x80000000UL)	/* R/WC */

#define		INT_STS_TXSTOP_INT_	(0x02000000UL)	/* R/WC */
#define		INT_STS_RXSTOP_INT_	(0x01000000UL)	/* R/WC */

#define		INT_STS_RXDFH_INT_	(0x00800000UL)	/* R/WC */
#define		INT_STS_TX_IOC_		(0x00200000UL)	/* R/WC */
#define		INT_STS_RXD_INT_		(0x00100000UL)	/* R/WC */

#define		INT_STS_GPT_INT_		(0x00080000UL)	/* R/WC */
#define		INT_STS_PHY_INT_		(0x00040000UL)	/* RO */
#define		INT_STS_PME_INT_		(0x00020000UL)	/* R/WC */
#define		INT_STS_TXSO_			(0x00010000UL)	/* R/WC */

#define		INT_STS_RWT_			(0x00008000UL)	/* R/WC */
#define		INT_STS_RXE_			(0x00004000UL)	/* R/WC */
#define		INT_STS_TXE_			(0x00002000UL)	/* R/WC */

#define		INT_STS_TDFO_			(0x00000400UL)	/* R/WC */
#define		INT_STS_TDFA_			(0x00000200UL)	/* R/WC */
#define		INT_STS_TSFF_			(0x00000100UL)	/* R/WC */

#define		INT_STS_TSFL_			(0x00000080UL)	/* R/WC */
#define		INT_STS_RDFO_			(0x00000040UL)	/* R/WC */
#define		INT_STS_RSFF_			(0x00000010UL)	/* R/WC */

#define		INT_STS_RSFL_			(0x00000008UL)	/* R/WC */
#define		INT_STS_GPIO2_INT_	(0x00000004UL)	/* R/WC */
#define		INT_STS_GPIO1_INT_	(0x00000002UL)	/* R/WC */
#define		INT_STS_GPIO0_INT_	(0x00000001UL)	/* R/WC */

#define INT_EN				(0x5CUL)
#define		INT_EN_SW_INT_EN_		(0x80000000UL)	/* R/W */
#define		INT_EN_TXSTOP_INT_EN_	(0x02000000UL)	/* R/W */
#define		INT_EN_RXSTOP_INT_EN_	(0x01000000UL)	/* R/W */
#define		INT_EN_RXDFH_INT_EN_		(0x00800000UL)	/* R/W */
#define		INT_EN_TIOC_INT_EN_		(0x00200000UL)	/* R/W */
#define		INT_EN_RXD_INT_EN_		(0x00100000UL)	/* R/W */
#define		INT_EN_GPT_INT_EN_		(0x00080000UL)	/* R/W */
#define		INT_EN_PHY_INT_EN_		(0x00040000UL)	/* R/W */
#define		INT_EN_PME_INT_EN_		(0x00020000UL)	/* R/W */
#define		INT_EN_TXSO_EN_			(0x00010000UL)	/* R/W */
#define		INT_EN_RWT_EN_			(0x00008000UL)	/* R/W */
#define		INT_EN_RXE_EN_			(0x00004000UL)	/* R/W */
#define		INT_EN_TXE_EN_			(0x00002000UL)	/* R/W */
#define		INT_EN_TDFO_EN_			(0x00000400UL)	/* R/W */
#define		INT_EN_TDFA_EN_			(0x00000200UL)	/* R/W */
#define		INT_EN_TSFF_EN_			(0x00000100UL)	/* R/W */
#define		INT_EN_TSFL_EN_			(0x00000080UL)	/* R/W */
#define		INT_EN_RDFO_EN_			(0x00000040UL)	/* R/W */
#define		INT_EN_RSFF_EN_			(0x00000010UL)	/* R/W */
#define		INT_EN_RSFL_EN_			(0x00000008UL)	/* R/W */
#define		INT_EN_GPIO2_INT_			(0x00000004UL)	/* R/W */
#define		INT_EN_GPIO1_INT_			(0x00000002UL)	/* R/W */
#define		INT_EN_GPIO0_INT_			(0x00000001UL)	/* R/W */

#define BYTE_TEST				(0x64UL)
#define FIFO_INT				(0x68UL)
#define		FIFO_INT_TX_AVAIL_LEVEL_	(0xFF000000UL)	/* R/W */
#define		FIFO_INT_TX_STS_LEVEL_		(0x00FF0000UL)	/* R/W */
#define		FIFO_INT_RX_STS_LEVEL_		(0x000000FFUL)	/* R/W */

#define RX_CFG					(0x6CUL)
#define		RX_CFG_RX_END_ALGN_		(0xC0000000UL)	/* R/W */
#define			RX_CFG_RX_END_ALGN4_		(0x00000000UL)	/* R/W */
#define			RX_CFG_RX_END_ALGN16_		(0x40000000UL)	/* R/W */
#define			RX_CFG_RX_END_ALGN32_		(0x80000000UL)	/* R/W */
#define		RX_CFG_RX_DMA_CNT_		(0x0FFF0000UL)	/* R/W */
#define		RX_CFG_RX_DUMP_			(0x00008000UL)	/* R/W */
#define		RX_CFG_RXDOFF_			(0x00001F00UL)	/* R/W */
#define 		RX_CFG_RXDOFF_2_			(0x00000200UL)    /* Rx data offset is 2 */
#define 		RX_CFG_RXDOFF_18_			(0x00001200UL)    /* Rx data offset is 0x12 */

#define TX_CFG					(0x70UL)
#define		TX_CFG_TXS_DUMP_		(0x00008000UL)	/* Self Clearing */
#define		TX_CFG_TXD_DUMP_		(0x00004000UL)	/* Self Clearing */
#define		TX_CFG_TXSAO_			(0x00000004UL)	/* R/W */
#define		TX_CFG_TX_ON_			(0x00000002UL)	/* R/W */
#define		TX_CFG_STOP_TX_			(0x00000001UL)	/* Self Clearing */

#define HW_CFG					(0x74UL)
#define 	HW_CFG_AMDIX_EN_			(0x01000000UL)	/*R0*/
#define		HW_CFG_MBO_					(0x00100000UL)	/* R/W */
#define		HW_CFG_TX_FIF_SZ_			(0x000F0000UL)	/* R/W */
#define     HW_CFG_PHY_CLK_SEL_			(0x00000060UL)  /* R/W */
#define     HW_CFG_PHY_CLK_SEL_INT_PHY_	(0x00000000UL) /* R/W */
#define     HW_CFG_PHY_CLK_SEL_EXT_PHY_	(0x00000020UL) /* R/W */
#define     HW_CFG_PHY_CLK_SEL_CLK_DIS_ (0x00000040UL) /* R/W */
#define     HW_CFG_SMI_SEL_				(0x00000010UL)  /* R/W */
#define     HW_CFG_EXT_PHY_DET_			(0x00000008UL)  /* RO */
#define     HW_CFG_EXT_PHY_EN_			(0x00000004UL)  /* R/W */
#define		HW_CFG_32_16_BIT_MODE_		(0x00000004UL)	/* RO */
#define     HW_CFG_SRST_TO_				(0x00000002UL)  /* RO */
#define		HW_CFG_SRST_				(0x00000001UL)	/* Self Clearing */

#define RX_DP_CTRL				(0x78UL)
#define		RX_DP_CTRL_FFWD_BUSY_	(0x80000000UL)	/* RO */

#define RX_FIFO_INF				(0x7CUL)
#define		RX_FIFO_INF_RXSUSED_	(0x00FF0000UL)	/* RO */
#define		RX_FIFO_INF_RXDUSED_	(0x0000FFFFUL)	/* RO */

#define TX_FIFO_INF				(0x80UL)
#define		TX_FIFO_INF_TSUSED_		(0x00FF0000UL)  /* RO */
#define		TX_FIFO_INF_TSFREE_		(0x00FF0000UL)	/* RO */
#define		TX_FIFO_INF_TDFREE_		(0x0000FFFFUL)	/* RO */

#define PMT_CTRL				(0x84UL)
#define		PMT_CTRL_PM_MODE_			(0x00003000UL)	/* Self Clearing */
#define		PMT_CTRL_PHY_RST_			(0x00000400UL)	/* Self Clearing */
#define		PMT_CTRL_WOL_EN_			(0x00000200UL)	/* R/W */
#define		PMT_CTRL_ED_EN_				(0x00000100UL)	/* R/W */
#define		PMT_CTRL_PME_TYPE_			(0x00000040UL)	/* R/W Not Affected by SW Reset */
#define		PMT_CTRL_WUPS_				(0x00000030UL)	/* R/WC */
#define			PMT_CTRL_WUPS_NOWAKE_		(0x00000000UL)	/* R/WC */
#define			PMT_CTRL_WUPS_ED_			(0x00000010UL)	/* R/WC */
#define			PMT_CTRL_WUPS_WOL_			(0x00000020UL)	/* R/WC */
#define			PMT_CTRL_WUPS_MULTI_		(0x00000030UL)	/* R/WC */
#define		PMT_CTRL_PME_IND_		(0x00000008UL)	/* R/W */
#define		PMT_CTRL_PME_POL_		(0x00000004UL)	/* R/W */
#define		PMT_CTRL_PME_EN_		(0x00000002UL)	/* R/W Not Affected by SW Reset */
#define		PMT_CTRL_READY_			(0x00000001UL)	/* RO */

#define GPIO_CFG				(0x88UL)
#define		GPIO_CFG_LED3_EN_		(0x40000000UL)	/* R/W */
#define		GPIO_CFG_LED2_EN_		(0x20000000UL)	/* R/W */
#define		GPIO_CFG_LED1_EN_		(0x10000000UL)	/* R/W */
#define		GPIO_CFG_GPIO2_INT_POL_	(0x04000000UL)	/* R/W */
#define		GPIO_CFG_GPIO1_INT_POL_	(0x02000000UL)	/* R/W */
#define		GPIO_CFG_GPIO0_INT_POL_	(0x01000000UL)	/* R/W */
#define		GPIO_CFG_EEPR_EN_		(0x00E00000UL)	/* R/W */
#define		GPIO_CFG_GPIOBUF2_		(0x00040000UL)	/* R/W */
#define		GPIO_CFG_GPIOBUF1_		(0x00020000UL)	/* R/W */
#define		GPIO_CFG_GPIOBUF0_		(0x00010000UL)	/* R/W */
#define		GPIO_CFG_GPIODIR2_		(0x00000400UL)	/* R/W */
#define		GPIO_CFG_GPIODIR1_		(0x00000200UL)	/* R/W */
#define		GPIO_CFG_GPIODIR0_		(0x00000100UL)	/* R/W */
#define		GPIO_CFG_GPIOD4_		(0x00000010UL)	/* R/W */
#define		GPIO_CFG_GPIOD3_		(0x00000008UL)	/* R/W */
#define		GPIO_CFG_GPIOD2_		(0x00000004UL)	/* R/W */
#define		GPIO_CFG_GPIOD1_		(0x00000002UL)	/* R/W */
#define		GPIO_CFG_GPIOD0_		(0x00000001UL)	/* R/W */

#define GPO4	GPIO_CFG_GPIOD4_ 
#define GPO3	GPIO_CFG_GPIOD3_ 
#define GPIO2	GPIO_CFG_GPIOD2_
#define GPIO1	GPIO_CFG_GPIOD1_
#define GPIO0	GPIO_CFG_GPIOD0_
#define GPIOs 	((UINT32) (GPO4 | GPO3 | GPIO2 | GPIO1 | GPIO0 ))

#define GPT_CFG					(0x8CUL)
#define		GPT_CFG_TIMER_EN_		(0x20000000UL)	/* R/W */
#define		GPT_CFG_GPT_LOAD_		(0x0000FFFFUL)	/* R/W */

#define GPT_CNT					(0x90UL)
#define		GPT_CNT_GPT_CNT_		(0x0000FFFFUL)	/* RO */

#define FPGA_REV				(0x94UL)
#define		FPGA_REV_FPGA_REV_		(0x0000FFFFUL)	/* RO */

#define ENDIAN					(0x98UL)
#define FREE_RUN				(0x9CUL)
#define RX_DROP					(0xA0UL)
#define MAC_CSR_CMD				(0xA4UL)
#define		MAC_CSR_CMD_CSR_BUSY_	(0x80000000UL)	/* Self Clearing */
#define		MAC_CSR_CMD_R_NOT_W_	(0x40000000UL)	/* R/W */
#define		MAC_CSR_CMD_CSR_ADDR_	(0x000000FFUL)	/* R/W */

#define MAC_CSR_DATA			(0xA8UL)
#define AFC_CFG					(0xACUL)
#define		AFC_CFG_AFC_HI_			(0x00FF0000UL)	/* R/W */
#define		AFC_CFG_AFC_LO_			(0x0000FF00UL)	/* R/W */
#define		AFC_CFG_BACK_DUR_		(0x000000F0UL)	/* R/W */
#define		AFC_CFG_FCMULT_			(0x00000008UL)	/* R/W */
#define		AFC_CFG_FCBRD_			(0x00000004UL)	/* R/W */
#define		AFC_CFG_FCADD_			(0x00000002UL)	/* R/W */
#define		AFC_CFG_FCANY_			(0x00000001UL)	/* R/W */

#define E2P_CMD					(0xB0UL)
#define		E2P_CMD_EPC_BUSY_		(0x80000000UL)	/* Self Clearing */
#define		E2P_CMD_EPC_CMD_		(0x70000000UL)	/* R/W */
#define			E2P_CMD_EPC_CMD_READ_	(0x00000000UL)	/* R/W */
#define			E2P_CMD_EPC_CMD_EWDS_	(0x10000000UL)	/* R/W */
#define			E2P_CMD_EPC_CMD_EWEN_	(0x20000000UL)	/* R/W */
#define			E2P_CMD_EPC_CMD_WRITE_	(0x30000000UL)	/* R/W */
#define			E2P_CMD_EPC_CMD_WRAL_	(0x40000000UL)	/* R/W */
#define			E2P_CMD_EPC_CMD_ERASE_	(0x50000000UL)	/* R/W */
#define			E2P_CMD_EPC_CMD_ERAL_	(0x60000000UL)	/* R/W */
#define			E2P_CMD_EPC_CMD_RELOAD_	(0x70000000UL)  /* R/W */
#define		E2P_CMD_EPC_TIMEOUT_	(0x00000200UL)	/* R */
#define		E2P_CMD_MAC_ADDR_LOADED_	(0x00000100UL)	/* RO */
#define		E2P_CMD_EPC_ADDR_		(0x000000FFUL)	/* R/W */

#define E2P_DATA				(0xB4UL)
#define		E2P_DATA_EEPROM_DATA_	(0x000000FFUL)	/* R/W */
/*end of lan register offsets and bit definitions */

#define LAN_REGISTER_EXTENT		(0x00002000UL)

#define LINK_OFF				(0x00UL)
#define LINK_SPEED_10HD			(0x01UL)
#define LINK_SPEED_10FD			(0x02UL)
#define LINK_SPEED_100HD		(0x04UL)
#define LINK_SPEED_100FD		(0x08UL)
#define LINK_SYMMETRIC_PAUSE	(0x10UL)
#define LINK_ASYMMETRIC_PAUSE	(0x20UL)
#define LINK_AUTO_NEGOTIATE		(0x40UL)

#define MDIX_DISABLE_SRT_CABLE 	(0X00UL) /*  1 0 0 ( bits 15 14 and 13 of phy register index 27*/
#define MDIX_DISABLE_CROSS_CABLE 	(0X01UL)  /*  1 0 1 ( bits 15 14 and 13 of phy register index 27*/
#define MDIX_AUT0_ENABLE			(0x02UL) /*  1 1 0 ( bits 15 14 and 13 of phy register index 27*/
#define MDIX_STRAP_CTRL			(0x03UL)


#define DRIVER_VERSION	(0x00000300UL)

/*
 ****************************************************************************
 *	Chip Specific MII Defines
 ****************************************************************************
 *
 *	Phy register offsets and bit definitions
 *
 */
#define LAN9118_PHY_ID	(0x00C0001C)

/* PHY BASIC CONTROL REGISTER*/
#define PHY_BCR			((UINT32)0U)
#define PHY_BCR_RESET_					((UINT16)0x8000U)
#define PHY_BCR_LOOPBACK_				((UINT16)0x4000U)
#define PHY_BCR_SPEED_SELECT_			((UINT16)0x2000U)
#define PHY_BCR_AUTO_NEG_ENABLE_		((UINT16)0x1000U)
#define PHY_BCR_RESTART_AUTO_NEG_	((UINT16)0x0200U)
#define PHY_BCR_DUPLEX_MODE_			((UINT16)0x0100U)

/* PHY BASIC STATUS REGISTER*/
#define PHY_BSR			((UINT32)1U)
#define PHY_BSR_LINK_STATUS_			((UINT16)0x0004U)
#define PHY_BSR_REMOTE_FAULT_			((UINT16)0x0010U)
#define PHY_BSR_AUTO_NEG_COMP_		((UINT16)0x0020U)


/* PHY IDENTIFIER 1 & 2*/

#define PHY_ID_1		((UINT32)2U)
#define PHY_ID_2		((UINT32)3U)

/* AUTO Negotitation Advertisement*/
#define PHY_ANEG_ADV    ((UINT32)4U)

#define PHY_ANEG_ADV_PAUSE_ 		((UINT16)0x0C00)
#define PHY_ANEG_ADV_ASYMP_			((UINT16)0x0800)
#define PHY_ANEG_ADV_SYMP_			((UINT16)0x0400)
#define PHY_ANEG_ADV_10H_			((UINT16)0x0020)
#define PHY_ANEG_ADV_10F_			((UINT16)0x0040)
#define PHY_ANEG_ADV_100H_			((UINT16)0x0080)
#define PHY_ANEG_ADV_100F_			((UINT16)0x0100)
#define PHY_ANEG_ADV_SPEED_			((UINT16)0x01E0)

/*Auto-negotiation Link Partner Ability*/
#define PHY_ANEG_LPA	((UINT32)5U)
#define PHY_ANEG_LPA_ASYMP_			((UINT16)0x0800)
#define PHY_ANEG_LPA_SYMP_			((UINT16)0x0400)
#define PHY_ANEG_LPA_100FDX_		((UINT16)0x0100)
#define PHY_ANEG_LPA_100HDX_		((UINT16)0x0080)
#define PHY_ANEG_LPA_10FDX_			((UINT16)0x0040)
#define PHY_ANEG_LPA_10HDX_			((UINT16)0x0020)

/* Mode Control/Status Register */

#define PHY_MODE_CTRL_STS		((UINT32)17)	
#define MODE_CTRL_STS_EDPWRDOWN_	((UINT16)0x2000U)
#define MODE_CTRL_STS_ENERGYON_		((UINT16)0x0002U)


/* Special Mode/ status Indications Register-921x */
#define PHY_SPLMODE_STSIND	((UINT32)27)

#define SPLMODE_STSIND_OVRIDE_AMDIX_		((UINT16)0x8000U)
#define SPLMODE_STSIND_AMIDX_EN_			((UINT16)0x4000U)
#define SPLMODE_STSIND_AMDIX_STS_		((UINT16)0x2000U)
#define SPLMODE_STSIND_VCOOFF_LP_		((UINT16)0x0400U)
#define SPLMODE_STSIND_XPOL_				((UINT16)0x0010U)


/*Interrupt Source Flag*/

#define PHY_INT_SRC			((UINT32)29)
#define PHY_INT_SRC_ENERGY_ON_			((UINT16)0x0080U)
#define PHY_INT_SRC_ANEG_COMP_			((UINT16)0x0040U)
#define PHY_INT_SRC_REMOTE_FAULT_		((UINT16)0x0020U)
#define PHY_INT_SRC_LINK_DOWN_			((UINT16)0x0010U)

/*Interrupt Mask*/

#define PHY_INT_MASK		((UINT32)30)
#define PHY_INT_MASK_ENERGY_ON_			((UINT16)0x0080U)
#define PHY_INT_MASK_ANEG_COMP_			((UINT16)0x0040U)
#define PHY_INT_MASK_REMOTE_FAULT_		((UINT16)0x0020U)
#define PHY_INT_MASK_LINK_DOWN_			((UINT16)0x0010U)

/*PHY Special Control/Status*/

#define PHY_SPECIAL			((UINT32)31)
#define PHY_SPECIAL_SPD_				((UINT16)0x001CU)
#define PHY_SPECIAL_SPD_10HALF_			((UINT16)0x0004U)
#define PHY_SPECIAL_SPD_10FULL_			((UINT16)0x0014U)
#define PHY_SPECIAL_SPD_100HALF_		((UINT16)0x0008U)
#define PHY_SPECIAL_SPD_100FULL_		((UINT16)0x0018U)

#define AMDIX_DISABLE_STRAIGHT			((UINT16)0x0U)
#define AMDIX_DISABLE_CROSSOVER  		((UINT16)0x01U)
#define AMDIX_ENABLE	                ((UINT16)0x02U)

#define LAN9118_DEV_NAME	"smsc"
#define LAN9118_DEV_NAME_LEN	5
#define LAN9118_EADR_LEN	6	
#define LAN9118_SPEED		10000000	/* 10Mbps or 100Mbps */
#define LAN9118_CRC_POLY	0x04c11db7

#define IMR_DISABLE		0
#define IMR_ENABLE		(IMR_PRXE | IMR_OVWE | IMR_PTXE | IMR_TXEE)
#define IMR_RX_DISABLE		(IMR_OVWE | IMR_PTXE | IMR_TXEE)
#define LAN9118_CSBASE		(0xB4000000)

/* debug macros */


#define 	LAN9118_DEBUG_OFF				0x0000000
#define 	LAN9118_DEBUG_RX				0x0000001
#define		LAN9118_DEBUG_TX				0x0000002
#define 	LAN9118_DEBUG_INT				0x0000004
#define		LAN9118_DEBUG_POLL_RX			0x0000008
#define		LAN9118_DEBUG_POLL_TX			0x0000010
#define		LAN9118_DEBUG_POLL				(LAN9118_DEBUG_POLL_RX | LAN9118_DEBUG_POLL_TX)
#define 	LAN9118_DEBUG_RX_ERR			0x0000020
#define		LAN9118_DEBUG_TX_ERR			0x0000040
#define 	LAN9118_DEBUG_RX_DATA_DUMP		0x0000080
#define		LAN9118_DEBUG_LOAD				0x0000100
#define		LAN9118_DEBUG_IOCTL				0x0000200
#define 	LAN9118_DEBUG_LOAD_ERROR		0x0000400
#define 	LAN9118_DEBUG_TRACE				0x0000800
#define 	LAN9118_DEBUG_POLL_REDIR		0x0010000
#define		LAN9118_DEBUG_LOG_NVRAM			0x0020000
#define		LAN9118_DEBUG_MB				0x0040000
#define		LAN9118_DEBUG_TMP				0x0080000
#define 	LAN9118_DEBUG_DUMP_REG			0x0100000
#define 	LAN9118_DEBUG_GPIO				0x0200000
#define 	LAN9118_DEBUG_MAC				0x0400000
#define 	LAN9118_DEBUG_PHY				0x0800000
#define 	LAN9118_DEBUG_ASSERT			0x1000000
#define 	LAN9118_DEBUG_WARNING			0x2000000

#ifdef	LAN9118_DEBUG

#define SET_GPIO(val) Lan_SetGPIO(pDrvCtrl, val)
#define CLR_GPIO(val) Lan_ClrGPIO(pDrvCtrl, val)
#define PULSE_GPIO(val) Lan_SetGPIO(pDrvCtrl, val);Lan_ClrGPIO(pDrvCtrl, val);

#define GPIO_ISR		GPIO0
#define GPIO_RX_COPY	GPIO1
#define GPIO_TX_COPY	GPIO2
#define GPIO_RX_JOB		GPO3
#define GPIO_RX_ERR		GPO4

#define DRV_LOG(FLG, X0, X1, X2, X3, X4, X5, X6)			\
	    if (LAN9118Debug_mode & FLG)						\
		logMsg(X0, X1, X2, X3, X4, X5, X6);

#else /* LAN9118_DEBUG */

#define DRV_LOG(DBG_SW, X0, X1, X2, X3, X4, X5, X6)
#define SET_GPIO(val) 
#define CLR_GPIO(val) 
#define PULSE_GPIO(val)

#endif /* LAN9118_DEBUG */

#ifdef	LOCAL
#undef	LOCAL
#define LOCAL
#endif

/* go to LAN9118End.h */
#define PLATFORM_CACHE_LINE_BYTES (16)

#define PLATFORM_IRQ_POL	(0)	/* IRQ Polarity Active_LOW=  Active_high=1*/
#define PLATFORM_IRQ_TYPE	(0)	/* Open_Drain = Push_pull= */

#define PLATFORM_IRQ		(32)
#define PLATFORM_RX_DMA		(2)
#define PLATFORM_TX_DMA		(3)
/* the dma threshold has not been thoroughly tuned but it is */
/*  slightly better than using one */
#define PLATFORM_DMA_THRESHOLD	100
#define PLATFORM_NAME		"SH3/SE01"

#define DEFAULT_RX_SRVICE_LOOP_COUNTER   1

/*
 ****************************************************************************
 *	Cache stuff
 ****************************************************************************
 *
 */
#define CACHE_PHYSADDR_MASK		0xFFFFFC00UL
#define CACHE_OC_ADDRESS_ARRAY	0xF0000000UL
#define CACHE_OC_ENTRY_MASK		0x00000FF0UL
#define CACHE_OC_ASSOC_BIT_		0x00000008UL

/**************************/
/*      12.BSC            */
/**************************/
#define	BCR1		(*(volatile unsigned short *)0xFFFFFF60)
#define	BCR2		(*(volatile unsigned short *)0xFFFFFF62)
#define	WCR1		(*(volatile unsigned short *)0xFFFFFF64)
#define	WCR2		(*(volatile unsigned short *)0xFFFFFF66)
 /* Bus Wait State Control Definitions */
 #define	WAIT_STATES_0		0x0
 #define	WAIT_STATES_1		0x1
 #define	WAIT_STATES_2		0x2
 #define	WAIT_STATES_3		0x3
 #define	WAIT_STATES_4		0x4
 #define	WAIT_STATES_6		0x5
 #define	WAIT_STATES_8		0x6
 #define	WAIT_STATES_10	0x7

 /* Bit map in WCR2 */
 #define	AREA_0				0
 #define	AREA_2				2
 #define	AREA_3				3
 #define	AREA_4				4
 #define	AREA_5				5
 #define	AREA_6				6

 #define	AREA0_WAIT_CTRL_	0x0007
 #define	AREA2_WAIT_CTRL_	0x0018
 #define	AREA3_WAIT_CTRL_	0x0060
 #define	AREA4_WAIT_CTRL_	0x0380
 #define	AREA5_WAIT_CTRL_	0x1C00
 #define	AREA6_WAIT_CTRL_	0xE000

/* Bit map of CHCRx */
 #define	DMAC_ENBL_		0x00000001UL

/**************************/
/*      14.DMAC           */
/**************************/
/*
	Chanel 0
*/
#define	SAR0		(*(volatile unsigned long *)0xA4000020UL)
#define	DAR0		(*(volatile unsigned long *)0xA4000024UL)
#define	DMATCR0		(*(volatile unsigned long *)0xA4000028UL)
#define	CHCR0		(*(volatile unsigned long *)0xA400002CUL)

/*
	Chanel 1
*/
#define	SAR1		(*(volatile unsigned long *)0xA4000030UL)
#define	DAR1		(*(volatile unsigned long *)0xA4000034UL)
#define	DMATCR1		(*(volatile unsigned long *)0xA4000038UL)
#define	CHCR1		(*(volatile unsigned long *)0xA400003CUL)

/*
	Chanel 2
*/
#define	SAR2		(*(volatile unsigned long *)0xA4000040UL)
#define	DAR2		(*(volatile unsigned long *)0xA4000044UL)
#define	DMATCR2		(*(volatile unsigned long *)0xA4000048UL)
#define	CHCR2		(*(volatile unsigned long *)0xA400004CUL)

/*
	Chanel 3
*/
#define	SAR3		(*(volatile unsigned long *)0xA4000050UL)
#define	DAR3		(*(volatile unsigned long *)0xA4000054UL)
#define	DMATCR3		(*(volatile unsigned long *)0xA4000058UL)
#define	CHCR3		(*(volatile unsigned long *)0xA400005CUL)

/* Bit map of CHCRx */
 #define	DMAC_ENBL_		0x00000001UL
 #define	XFER_END_		0x00000002UL
 #define	INT_ENBL_		0x00000004UL

 #define	XMIT_SIZE_		0x00000018UL
 #define	XMIT_BYTE		0x00000000UL
 #define	XMIT_WORD		0x00000008UL
 #define	XMIT_DWORD		0x00000010UL
 #define	XMIT_QUINT32	0x00000018UL

 #define	XMIT_MODE_			0x00000020UL

 #define	XMIT_RESRC_			0x00000F00UL
 #define	AUTO_REQ				0x00000400UL

 #define	SRC_ADDR_MODE_		0x00003000UL
 #define	FIXED_SRC_ADDR		0x00000000UL
 #define	INC_SRC_ADDR		0x00001000UL
 #define	DEC_SRC_ADDR		0x00002000UL

 #define	DST_ADDR_MODE_		0x0000C000UL
 #define	FIXED_DST_ADDR		0x00000000UL
 #define	INC_DST_ADDR		0x00004000UL
 #define	DEC_DST_ADDR		0x00008000UL

/*
	Common
*/
#define	DMAOR		(*(volatile unsigned short *)0xA4000060UL)
 #define	DMA_MSTR_EN_		0x0001
 #define	NMI_FLAG_			0x0002
 #define	ADDR_ERR_FLAG_	0x0004

#define	CHRAR		(*(volatile unsigned short *)0xA400022AUL)

/*
 ****************************************************************************
 ****************************************************************************
 *	MAC Control and Status Register (Indirect Address)
 *	Offset (through the MAC_CSR CMD and DATA port)
 ****************************************************************************
 ****************************************************************************
 *
 */
#define MAC_CR				(0x01UL)	/* R/W */

	/* MAC_CR - MAC Control Register */
	#define MAC_CR_RXALL_		(0x80000000UL)
	#define MAC_CR_HBDIS_		(0x10000000UL)
	#define MAC_CR_RCVOWN_		(0x00800000UL)
	#define MAC_CR_LOOPBK_		(0x00200000UL)
	#define MAC_CR_FDPX_		(0x00100000UL)
	#define MAC_CR_MCPAS_		(0x00080000UL)
	#define MAC_CR_PRMS_		(0x00040000UL)
	#define MAC_CR_INVFILT_		(0x00020000UL)
	#define MAC_CR_PASSBAD_		(0x00010000UL)
	#define MAC_CR_HFILT_		(0x00008000UL)
	#define MAC_CR_HPFILT_		(0x00002000UL)
	#define MAC_CR_LCOLL_		(0x00001000UL)
	#define MAC_CR_BCAST_		(0x00000800UL)
	#define MAC_CR_DISRTY_		(0x00000400UL)
	#define MAC_CR_PADSTR_		(0x00000100UL)
	#define MAC_CR_BOLMT_MASK_	(0x000000C0UL)
	#define MAC_CR_DFCHK_		(0x00000020UL)
	#define MAC_CR_TXEN_		(0x00000008UL)
	#define MAC_CR_RXEN_		(0x00000004UL)

#define ADDRH				(0x02UL)	/* R/W mask 0x0000FFFFUL */
#define ADDRL				(0x03UL)	/* R/W mask 0xFFFFFFFFUL */
#define HASHH				(0x04UL)	/* R/W */
#define HASHL				(0x05UL)	/* R/W */

#define MII_ACC				(0x06UL)	/* R/W */
	#define MII_ACC_PHY_ADDR_	(0x0000F800UL)
	#define MII_ACC_MIIRINDA_	(0x000007C0UL)
	#define MII_ACC_MII_WRITE_	(0x00000002UL)
	#define MII_ACC_MII_BUSY_	(0x00000001UL)

#define MII_DATA			(0x07UL)	/* R/W mask 0x0000FFFFUL */

#define FLOW				(0x08UL)	/* R/W */
	#define FLOW_FCPT_			(0xFFFF0000UL)
	#define FLOW_FCPASS_		(0x00000004UL)
	#define FLOW_FCEN_			(0x00000002UL)
	#define FLOW_FCBSY_			(0x00000001UL)

#define VLAN1				(0x09UL)	/* R/W mask 0x0000FFFFUL */
#define VLAN2				(0x0AUL)	/* R/W mask 0x0000FFFFUL */

#define WUFF				(0x0BUL)	/* WO */

#define WUCSR				(0x0CUL)	/* R/W */
	#define WUCSR_GUE_			(0x00000200UL)
	#define WUCSR_WUFR_			(0x00000040UL)
	#define WUCSR_MPR_			(0x00000020UL)
	#define WUCSR_WAKE_EN_		(0x00000004UL)
	#define WUCSR_MPEN_			(0x00000002UL)

#define COE_CR				(0x0DUL)
	#define TX_COE_EN			(0x00010000UL)
	#define RX_COE_MODE			(0x00000002UL)
	#define RX_COE_EN			(0x00000001UL)


#define Lan_GetRegDW(dwOffset)	\
	Platform_GetRegDW(pDrvCtrl->dwLanBase,dwOffset)

#define Lan_SetRegDW(dwOffset,dwVal) \
	Platform_SetRegDW(pDrvCtrl->dwLanBase,dwOffset,dwVal)

#define Lan_ClrBitsDW(dwOffset,dwBits)			\
	Platform_SetRegDW(pDrvCtrl->dwLanBase,				\
		dwOffset,Platform_GetRegDW(pDrvCtrl->dwLanBase,	\
		dwOffset)&(~dwBits))

#define Lan_SetBitsDW(dwOffset,dwBits)			\
	Platform_SetRegDW(pDrvCtrl->dwLanBase,				\
		dwOffset,Platform_GetRegDW(pDrvCtrl->dwLanBase,	\
		dwOffset)|dwBits);
 
#define OLD_REGISTERS(pDrvCtrl)		\
	(((pDrvCtrl->dwIdRev==0x01180000)==0)&& \
	 ((pDrvCtrl->dwFpgaRev)>=0x01)&& \
	 ((pDrvCtrl->dwFpgaRev)<=0x25))

#define CpuToPhysicalAddr(cpuAddr) ((UINT32)cpuAddr)

#define IS_REV_A(pDrvCtrl)	((pDrvCtrl->dwIdRev&0x0000FFFF)==0)


#ifndef SYS_INT_CONNECT
#define SYS_INT_CONNECT(pDrvCtrl,rtn,arg,pResult) 			\
    do {								\
    *pResult = intConnect ((VOIDFUNCPTR *)INUM_TO_IVEC (pDrvCtrl->ivec),\
			     rtn, (int)arg); 			\
    } while (0)
#endif /* SYS_INT_CONNECT */

#ifndef SYS_INT_DISCONNECT
#define SYS_INT_DISCONNECT(pDrvCtrl,rtn,arg,pResult) 			\
    do { 								\
    *pResult = OK;	 					\
    } while (0)
#endif /* SYS_INT_DISCONNECT */

#ifndef SYS_INT_ENABLE
#define SYS_INT_ENABLE(pDrvCtrl) 		\
    do { 								\
    IMPORT STATUS sysIntEnablePIC(int);	\
    sysIntEnablePIC (pDrvCtrl->ilevel);	\
    } while (0)
#endif /* SYS_INT_ENABLE */

#ifndef SYS_INT_DISABLE
#define SYS_INT_DISABLE(pDrvCtrl)            	\
    do {                                  	    \
    IMPORT STATUS sysIntDisablePIC(int);		\
    sysIntDisablePIC(pDrvCtrl->ilevel);			\
    } while (0)
#endif /*SYS_INT_DISABLE*/

#ifndef SYS_MS_DELAY
#define SYS_MS_DELAY(delay)				\
    do {								\
    IMPORT void sysMsDelay (UINT);		\
    sysMsDelay (delay);					\
    } while (0)
#endif /* SYS_MS_DELAY */

/* Macros for dealing with flags */

#define DRV_FLAGS_SET(setBits)                 \
    (pDrvCtrl->flags |= (setBits))

#define DRV_FLAGS_ISSET(setBits)               \
    (pDrvCtrl->flags & (setBits))

#define DRV_FLAGS_CLR(clrBits)                 \
    (pDrvCtrl->flags &= ~(clrBits))

#define DRV_FLAGS_GET()                        \
    (pDrvCtrl->flags)

#define LAN9118_IS_IN_POLL_MODE()		\
    ((DRV_FLAGS_GET() & LAN9118_FLAG_POLL) == LAN9118_FLAG_POLL)

#define LAN9118_SEM_TAKE(pDrvCtrl, timeout)			\
    semTake ((pDrvCtrl)->endObj.txSem, timeout)

#define LAN9118_SEM_GIVE(pDrvCtrl)					\
    semGive((pDrvCtrl)->endObj.txSem)

/* A shortcut for getting the hardware address from the MIB II stuff. */

#define END_HADDR(pEnd)								\
    ((pEnd)->mib2Tbl.ifPhysAddress.phyAddress)

#define END_HADDR_LEN(pEnd) 						\
    ((pEnd)->mib2Tbl.ifPhysAddress.addrLength)

#define END_FLAGS_ISSET(pEnd, setBits) 				\
    ((pEnd)->flags & (setBits))

#define END_ADD_IF_INERR(pEnd, val)                 \
{                                                   \
	if ((pEnd)->pIf != NULL)                        \
	{                                               \
		spin_lock(&((pEnd)->ifCounterLock));        \
		(pEnd)->pIf->if_ierrors+=val;               \
		spin_unlock(&((pEnd)->ifCounterLock));      \
	}                                               \
}
#define END_ADD_IF_OUTERR(pEnd, val)                \
{                                                   \
	if ((pEnd)->pIf != NULL)                        \
	{                                               \
		spin_lock(&((pEnd)->ifCounterLock));        \
		(pEnd)->pIf->if_oerrors+=val;               \
		spin_unlock(&((pEnd)->ifCounterLock));      \
	}                                               \
}
#define END_ADD_IF_COL(pEnd, val)                   \
{                                                   \
	if ((pEnd)->pIf != NULL)                        \
	{                                               \
		spin_lock(&((pEnd)->ifCounterLock));        \
		(pEnd)->pIf->if_collisions+=val;            \
		spin_unlock(&((pEnd)->ifCounterLock));      \
	}                                               \
}
#define END_ADD_IF_DROP(pEnd, val)                  \
{                                                   \
	if ((pEnd)->pIf != NULL)                        \
	{                                               \
		spin_lock(&((pEnd)->ifCounterLock));        \
		(pEnd)->pIf->if_iqdrops+=val;               \
		spin_unlock(&((pEnd)->ifCounterLock));      \
	}                                               \
}

/*
 * spin lock macros.
 *
 */
#define spin_lock_init(psemLock)					\
{													\
	SMSC_ASSERT(psemLock!=NULL);					\
	(* (SEM_ID *)psemLock) = semMCreate (SEM_Q_PRIORITY | SEM_INVERSION_SAFE | SEM_DELETE_SAFE);	\
	if ((* (SEM_ID *)psemLock) == NULL) {			\
   	    DRV_LOG (LAN9118_DEBUG_WARNING, "The semaphore can not be created\n", 1, 2, 3, 4, 5, 6);	\
	}												\
}
#define spin_lock(psemLock)							\
{													\
	SMSC_ASSERT(psemLock!=NULL);					\
	SMSC_ASSERT((* (SEM_ID *)psemLock)!=NULL)		\
	semTake ((* (SEM_ID *)psemLock), WAIT_FOREVER);	\
}	
#define spin_unlock(psemLock)						\
{													\
	SMSC_ASSERT(psemLock!=NULL);					\
	SMSC_ASSERT((* (SEM_ID *)psemLock)!=NULL);		\
	semGive ((* (SEM_ID *)psemLock));				\
}	
#define spin_lock_irqsave(psemLock, IntFlg)		    \
{													\
	SMSC_ASSERT(psemLock!=NULL);					\
	SMSC_ASSERT((* (SEM_ID *)psemLock)!=NULL);		\
	semTake ((* (SEM_ID *)psemLock), WAIT_FOREVER);	\
	IntFlg = intLock();							    \
}	
#define spin_unlock_irqrestore(psemLock, IntFlg)    \
{													\
	SMSC_ASSERT(psemLock!=NULL);					\
	SMSC_ASSERT((* (SEM_ID *)psemLock)!=NULL);		\
	intUnlock(IntFlg);							    \
	semGive ((* (SEM_ID *)psemLock));				\
}	

/*
 * Macros for read and write descriptors.
 *
 */

#define LAN9118_RX_BUF_SZ	sizeof(LAN9118_RX_FRAME)
#define LAN9118_ETH_CRC_LEN	4



/* typedefs */
typedef unsigned char BOOLEAN;
typedef unsigned char BYTE;

typedef long TIME_SPAN;

#define MAX_TIME_SPAN	((TIME_SPAN)(0x7FFFFFFFUL))

typedef struct _SMSC9118_IOCTL_DATA {
	unsigned long dwSignature;

	unsigned long dwCommand;

	unsigned long Data[0x60];

	char Strng1[30];

	char Strng2[10];

} SMSC9118_IOCTL_DATA, *PSMSC9118_IOCTL_DATA;

/*
 * Default macro definitions for BSP interface.
 * These macros can be redefined in a wrapper file, to generate
 * a new module with an optimized interface.
 */
/*
The following describes the synchronization policies used in this driver.
Register Name				Policy
RX_DATA_FIFO				Only used by the Rx Thread, Rx_ProcessPackets
TX_DATA_FIFO				Only used by the Tx Thread, Tx_SendSkb
RX_STATUS_FIFO			Only used by the Rx Thread, Rx_ProcessPackets
RX_STATUS_FIFO_PEEK		Not used.
TX_STATUS_FIFO			Used in	Tx_CompleteTx in Tx_UpdateTxCounters.
							protected by TxCounterLock
TX_STATUS_FIFO_PEEK		Not used.
ID_REV					Read only
INT_CFG					Set in Lan_Initialize, 
						protected by IntEnableLock
INT_STS					Sharable, 
INT_EN					Initialized at startup, 
							Used in Rx_ProcessPackets
							otherwise protected by IntEnableLock
BYTE_TEST					Read Only
FIFO_INT					Initialized at startup,
                          During run time only accessed by 
                              Tx_HandleInterrupt, and Tx_SendSkb and done in a safe manner
RX_CFG					Used during initialization
                          During runtime only used by Rx Thread
TX_CFG					Only used during initialization
HW_CFG					Only used during initialization
RX_DP_CTRL				Only used in Rx Thread, in Rx_FastForward
RX_FIFO_INF				Read Only, Only used in Rx Thread, in Rx_PopRxStatus
TX_FIFO_INF				Read Only, Only used in Tx Thread, in Tx_GetTxStatusCount, Tx_SendSkb, Tx_CompleteTx
PMT_CTRL					Not Used
GPIO_CFG					used during initialization, in Lan_Initialize
                          used for debugging
                          used during EEPROM access.
                          safe enough to not require a lock
GPT_CFG					protected by GpTimerLock
GPT_CNT					Not Used
ENDIAN					Not Used
FREE_RUN					Read only
RX_DROP					Used in Rx Interrupt Handler,
                          and get_stats.
                          safe enough to not require a lock.
MAC_CSR_CMD				Protected by MacPhyLock
MAC_CSR_DATA				Protected by MacPhyLock
                          Because the two previous MAC_CSR_ registers are protected
                            All MAC, and PHY registers are protected as well.
AFC_CFG					Used during initialization, in Lan_Initialize
                          During run time, used in timer call back, in Phy_UpdateLinkMode
E2P_CMD					Used during initialization, in Lan_Initialize
                          Used in EEPROM functions
E2P_DATA					Used in EEPROM functions
*/
#ifdef __cplusplus
}
#endif

#endif /* __INCLAN9118Endh */
